@@ -148,13 +148,13 @@ def job(job_name: str) -> Callable:
                 message = f"<b><u>ERROR</u></b>\n\nJob Name: <i>{job_name}</i>\nError Type: <i>{type(e).__name__}</i>"
                 if str(e) != "":
                     message = message + f"\nError Message: <i>{str(e).replace('<', '').replace('>', '')}</i>"
-                telegram.send_message(telegram.constants.telegram_debug_channel_username, message,
+                telegram.send_message(telegram.constants.telegram_channel_username, message,
                                                     True)
                 raise Exception(e)
             finally:
                 logger.debug(f"Job ended: {job_name}")
                 telegram.send_message(
-                    telegram.constants.telegram_channel_username,
+                    telegram.constants.telegram_debug_channel_username,
                     f"Job ended: <i>{job_name}</i>", True)
 
         return wrapper
